// closure to prevent namespace pollution
(function($) {
	// global application variable
	var APP = (window.APP = window.APP || {});
	// global goal related properties
	APP.Debug = false;
	APP.KeysKey = "goal.keys"; //part of the application so defined here but used in storage.js
	APP.CurrentGoalHash = "";
    APP.CurrentTemplate = "";
	APP.IsFreshlySaved = false;
	// html functinality
	APP.newGoalWorksheet = function(title, outcome) {
		return $('\
			<div data-goal="worksheet" class="worksheet">\
				<form>\
					<input type="hidden" data-goal="current" value="" />\
					<ul data-role="listview" data-theme="a" class="noline">\
						<li class="nopad">\
							<div class="nomar ui-field-contain">\
								<p>Goal Title</p>\
								<textarea data-goal="title" class="goal_title" id="goal_title" name="textarea" style="height: 50px;" placeholder="Name your goal">' + (title || "") + '</textarea>\
							</div>\
							<div class="nomar ui-field-contain">\
								<p>Goal Outcome</p>\
								<textarea data-goal="outcome" class="goal_outcome" id="goal_outcome" name="textarea" style="height: 50px;" placeholder="Describe the outcome">' + (outcome || "") + '</textarea>\
							</div>\
						</li>\
					</ul>\
                    <hr />\
					<div data-goal="sections"></div>\
					<div class="ui-grid-d ui-responsive">\
						<div class="nopad nomar ui-block">\
							<button type="button" class="add_p" style="height: 50px; width: 100%;" data-goal="addsection">Add Priority</button>\
						</div>\
					</div>\
				</form>\
			</div>');
	};
	
	// html functionality for display goals
	APP.displayGoalWorksheet = function(outcome) {
		return $('\
			<div data-goal="worksheet" class="worksheet">\
				<input type="hidden" data-goal="current" value="" />\
				<p><span class="hilite">Goal Outcome: </span> ' + (outcome || "") + '</p><br />\
				<div data-goal="sections"></div>\
			</div>');
	};
	APP.setGoalWorksheet = function($page, $worksheet) {
		$page.find("div[role='main']").empty().append($worksheet.enhanceWithin());
	};

	APP.newSection = function(index, priority) {
		return $('\
			<div data-goal="section">\
				<p class="subhead goal-subhead">Priority ' + (index || 1) + '</p>\
				<ul data-role="listview" data-theme="a">\
					<li class="nopad"><p>Priority</p>\
						<textarea data-goal="priority" style="height: 50px;">' + (priority || "") + '</textarea>\
					</li>\
					<li class="nopad"><p>Action Statements</p></li>\
				    <li class="nopad" data-goal="actions"></li>\
					<li class="nopad-top nopad-left">\
							<button type="button" class="ui-btn add_a" data-goal="addaction">Add Action Statement</button>\
					</li>\
					<li class="nopad">Power Statements</li>\
				    <li class="nopad" data-goal="beliefs">\
					<li class="nopad-top nopad-left">\
							<button type="button" class="ui-btn add_b" data-goal="addbelief">Add Power Statement</button>\
					</li>\
				</ul>\
                <hr />\
			</div>');
	};
	APP.displaySection = function(index, priority) {
		return $('\
			<div data-goal="section">\
				<hr />\
				<p><b class="hilite">Priority ' + (index || 1) + '</b></p>\
				<ul>\
                    <li>' + (priority || "") + '</li>\
                </ul>\
				<p class="hilite">Action Statements</p>\
                <ul data-goal="actions">\
                </ul>\
				<p class="hilite">Power Statements</p>\
                <ul data-goal="beliefs">\
                </ul>\
			</div>');
	};
	APP.addSection = function($worksheet, $section) {
		$worksheet.find("div:jqmData(goal='sections')").append($section.enhanceWithin());
	};
	APP.addAction = function($section, value) {
		$section.find("li:jqmData(goal='actions')")
			.append($('<textarea>' + (value || "") + '</textarea>'))
			.enhanceWithin();
	};
	
	APP.displayAction = function($section, value) {
		$section.find("ul:jqmData(goal='actions')")
			.append($('<li>' + (value || "") + '</li>'))
			.enhanceWithin();
	};
	APP.addBelief = function($section, value) {
		$section.find("li:jqmData(goal='beliefs')")
			.append($('<textarea>' + (value || "") + '</textarea>'))
			.enhanceWithin();
	};
	APP.displayBelief = function($section, value) {
		$section.find("ul:jqmData(goal='beliefs')")
			.append($('<li>' + (value || "") + '</li>'))
			.enhanceWithin();
	};
	APP.addFullSection = function($worksheet, index, priority, actions, beliefs) {
		// ensure values were given for each
		var index = index || 1,
			priority = priority || "",
			actions = actions || [],
			beliefs = beliefs || [];
		// force actions and beliefs to have at least one entry
		if (actions.length == 0) actions.push("");
		if (beliefs.length == 0) beliefs.push("");
		// start new section
		var $section = APP.newSection(index, priority);
		// add actions
		for (var i = 0; i < actions.length; i++)
			APP.addAction($section, actions[i]);
		// add beliefs
		for (var i = 0; i < beliefs.length; i++)
			APP.addBelief($section, beliefs[i]);
		// add the section to the worksheet
		APP.addSection($worksheet, $section);
	};
	APP.displayFullSection = function($worksheet, index, priority, actions, beliefs) {
		// safety: ensure values were given for each
		var index = index || 1,
			priority = priority || "",
			actions = actions || [],
			beliefs = beliefs || [];
		// start new section
		var $section = APP.displaySection(index, priority);
		// display actions
		for (var i = 0; i < actions.length; i++)
			APP.displayAction($section, actions[i]);
		// display beliefs
		for (var i = 0; i < beliefs.length; i++)
			APP.displayBelief($section, beliefs[i]);
		// add the section to the worksheet
		APP.addSection($worksheet, $section);
	};
	
// counting functions
	APP.getSectionCount = function($worksheet) {
		return $worksheet.find("div:jqmData(goal='sections')").children().length;
	};
	APP.getActionCount = function($section) {
		return $section.find("div:jqmData(goal='actions')").children().length;
	};
	APP.getBeliefCount = function($section) {
		return $section.find("div:jqmData(goal='beliefs')").children().length;
	};
// retrieval functions (getting data user typed in from html page)
	APP.getActions = function($section) {
		var actions = [];
		$section.find("li:jqmData(goal='actions')").children("textarea").each(function() {
			actions.push($(this).val().trim());
		});
		return actions;
	};
	APP.getBeliefs = function($section) {
		var beliefs = [];
		$section.find("li:jqmData(goal='beliefs')").children("textarea").each(function() {
			beliefs.push($(this).val().trim());
		});
		return beliefs;
	};
	APP.getPriority = function($section) {
		return $section.find("textarea:jqmData(goal='priority')").val() || "";
	};
	//gets the actions, beliefs, and priority for this section
	APP.getSection = function($section) {
		return {
			"priority": APP.getPriority($section).trim(),
			"actions": APP.getActions($section),
			"beliefs": APP.getBeliefs($section)
		};
	};
	///gets all the sections
	APP.getSections = function($worksheet) {
		var sections = [];
		$worksheet.find("div:jqmData(goal='sections')").children("div").each(function() {
			sections.push(APP.getSection($(this)));
		});
		return sections;
	};
	APP.getTitle = function($worksheet) {
		return $worksheet.find("textarea:jqmData(goal='title')").val() || "";
	};
	APP.getOutcome = function($worksheet) {
		return $worksheet.find("textarea:jqmData(goal='outcome')").val() || "";
	};
	APP.getWorksheetData = function($worksheet) {
		return {
			"title": APP.getTitle($worksheet).trim(),
			"outcome": APP.getOutcome($worksheet).trim(),
			"sections": APP.getSections($worksheet)
		};
	};
	
	// add/remove sections
	$(document).on("click", "button:jqmData(goal='addsection')", function() {
		var $worksheet = $(this).parents("div:jqmData(goal='worksheet')");
		APP.addFullSection($worksheet, APP.getSectionCount($worksheet) + 1, "", [""], [""]);
	});
	$(document).on("click", "button:jqmData(goal='delsection')", function() {
		var $sections = $(this)
			.parents("div:jqmData(goal='worksheet')")
			.find("div:jqmData(goal='sections')")
			.children();
		// ensure we don't remove the only section
		if ($sections.length < 2)
			return;
		// remove the last one
		$sections.last().remove();
	});
	
	// add/remove actions/beliefs in a dynamically generated section
	$(document).on("click", "button:jqmData(goal='addaction')", function() {
		var $section = $(this).parents("div:jqmData(goal='section')");
		APP.addAction($section, "");
	});
	
	$(document).on("click", "button:jqmData(goal='addbelief')", function() {
		var $section = $(this).parents("div:jqmData(goal='section')");
		APP.addBelief($section, "");
	});
	$(document).on("click", "button:jqmData(goal='deltext')", function() {
		// get the textarea children of li[data-goal]
		var $textareas = $(this)
			.parents("ul:jqmData(role='listview')")
			.children("li:jqmData(goal)")
			.children("textarea");
		// ensure we don't remove the only textarea
		if ($textareas.length < 2)
			return;
		// remove the last one
		$textareas.last().remove();
	});
	$(document).on("click", "button:jqmData(goal='clearall')", function() {
		APP.Storage.clearAll();
		$(":mobile-pagecontainer").pagecontainer("change", "#index");			
	});
	
})(jQuery);