// closure to prevent namespace polution
(function($) {
	// global application variable
	var APP = (window.APP = window.APP || {});
	// global storage functionality
	var Storage = APP.Storage = (window.APP.Storage = window.APP.Storage || {});
    
	//attach function to document on page initialization, target #new element to build empty goal worksheet

    //> convenience listener to handle freshly saved worksheets by intercepting the transition, and sending the user to #edit when coming from #new or #edit-template.
    $(document).on('pagecontainerbeforechange', function(e, ui) {
        // first, ensure we transitioning properly (both pages will be jquery objects)
        if (ui.toPage && ui.toPage.jquery && ui.prevPage && ui.prevPage.jquery) {
            var $toPage   = ui.toPage,
                $prevPage = ui.prevPage;
            // ensure we coming FROM the save dialog
            if ($prevPage.attr('id') == "save") {
                // ensure we are going TO either #new or #edit-template
                if ($toPage.attr('id') == "new" || $toPage.attr('id') == "edit-template") {
                    // if we get here while APP.IsFreshlySaved is set...
                    if (APP.IsFreshlySaved) {
                        // prevent page transition
                        e.preventDefault();
                        // redirect and exit
                        $(":mobile-pagecontainer").pagecontainer("change", "#edit", {
                            transition: "fade"
                        });
                        return false;
                    }
                }
            }
        }
    });
    
    $(document).on('pageinit', '#new', function(e, ui) {
		if (APP.Debug) console.log("initing #new!");
		// create a new worksheet
		var $worksheet = APP.newGoalWorksheet();
		// create a new section
		APP.addFullSection($worksheet, 1, "", [""], [""]);
		APP.setGoalWorksheet($(this), $worksheet);
	});
    
    $(document).on('pageshowing', '#new', function(e, ui) {        
		if (APP.Debug) console.log("showing #new!");
		// create a new worksheet
        var $page = $(ui.prevPage);
        // if we came from the list page, show a new worksheet
        if ($page.attr('id') == 'index') {
            var $worksheet = APP.newGoalWorksheet();
            // create a new section
            APP.addFullSection($worksheet, 1, "", [""], [""]);
            APP.setGoalWorksheet($(this), $worksheet);
        }
	});
    
	//attach function to document before it shows, target #save element to save the current/new goal
	$(document).on('pageshowing', '#save', function(e, ui) {
		if (APP.Debug) console.log("showing #save!");
		// grab the #new and #save page
		var $page = $(ui.prevPage);
		var $save = $(ui.toPage);
		// if the user came directly to this dialog via url, redirect them and exit
		if ($page.length == 0) {
			$(":mobile-pagecontainer").pagecontainer("change", "#new", { transition: "none" });
			return;
		}
		// get the worksheet and data-goal="save-msg"
		var $worksheet = $page.find("div:jqmData(goal='worksheet')");
		var $saveMsg = $save.find(":jqmData(goal='save-msg')");
		var data = APP.getWorksheetData($worksheet);
        
        // remove 'view worksheet' button, and clear IsFreshlySaved just in case
        $save.find('a[href="#view"]').remove();
        APP.IsFreshlySaved = false;
        
		// validation return message
		var validMsg = "";
		// if validation returns an empty string, something was wrong
		if ((validMsg = APP.Storage.validateWorksheetData(data)) !== "") {
			// replace the data-goal="save-msg" text
			$saveMsg.html(validMsg);
			return;
		}
		// save return message
		var saveResult;
        
        // if we're on the Edit Worksheet page...
        if ($page.attr('id') == 'edit') {
            if (APP.Debug) console.log("edit page #save");
            var oldKey = $page.find('[data-goal-old]').attr('data-goal-old');
            saveResult = APP.Storage.tryOverwriteWorksheetData(data, oldKey);
        } else {
            if (APP.Debug) console.log("new page #save");
            saveResult = APP.Storage.trySaveWorksheetData(data);
        }
        // if an error occurred
        if (saveResult !== "") {
            // show error message
            $saveMsg.html(saveResult);
            return;
        }
		// no errors - 
        $save.find('a[href="#"]').before($('<a href="#view" class="ui-btn ui-corner-all"  style="border-color:#F4F4F4; background: rgba(255,255,255,.05); margin: 0 !important; padding: 1em 4px !important;" >View Goals</a>'));
        // let the app know we just saved something
        APP.IsFreshlySaved = true;
		$saveMsg.html("Worksheet saved.");
	});
    
	//attach function to document on page initialization, target #list element to store the hash
	$(document).on('pageinit', '#list', function(e, ui) {
		if (APP.Debug) console.log("initing #list!");
		// grab the #list page
		var $list = $(this);
		//function for the list item
		$list.on('click', 'a[href="#view"]', function(e, ui) {
			APP.CurrentGoalHash = $(this).attr('data-goal'); //the key
		});
	});
    
    //attach function to document on page initialization, target #remove element
	$(document).on('pageinit', '#remove', function(e, ui) {
		if (APP.Debug) console.log("initing #remove!");
		// grab the #remove page
		var $remove = $(this);
		//function for the list item
		$remove.on('click', 'a[data-goal]', function(e, ui) {
			APP.Storage.clearWorksheetData($(this).attr('data-goal')); //the key
            var $parent = $(this).parent("li");
            if ($parent.length > 0) {
                var $ul = $parent.parent();
                // force fading items to go away immediately
                $ul.children().finish();
                // if there is only a single worksheet
                if ($ul.children().length == 1) {
                    // replace the li with the 'no goals' li
                    $parent.fadeOut(500, function() {
                        $(this).addClass('ui-li-static').empty().text('No goals to remove').fadeIn(500);
                    })
                }
                // else, remove the list item
                else {
                    $parent.fadeOut(500, function() {
                        $(this).remove();
                    });
                }
            }
		});
	});
    
    //attach function to document on page initialization, target #templates element to grab template id
	$(document).on('pageinit', '#templates', function(e, ui) {
		if (APP.Debug) console.log("initing #templates!");
		// grab the #list page
		var $templates = $(this);
		//function for the list item
		$templates.on('click', 'a[href="#edit-template"]', function(e, ui) {
			APP.CurrentTemplate = $(this).attr('data-template'); //the key
		});
	});
    
    //attach function to document on pageshowing, target #view element to store the hash
    $(document).on('pageshowing', '#view', function(e, ui) {
		if (APP.Debug) console.log("showing #view!");
		//if there is no previous page send them back to list
		if ($(ui.prevPage).length == 0) {
			$(":mobile-pagecontainer").pagecontainer("change", "#list", { transition: "none" });
			return;
		}
		if (APP.Debug) console.log(APP.CurrentGoalHash);
        
		var $view = $(ui.toPage);
		var data = Storage.getWorksheetData(APP.CurrentGoalHash);
		var $worksheet = APP.displayGoalWorksheet(data.outcome);
		$view.find('[data-role="header"]').children('h1').html(data.title);
		// display the sections
		for (var i = 0; i < data.sections.length; i++) {
			APP.displayFullSection($worksheet, i+1, data.sections[i].priority, data.sections[i].actions, data.sections[i].beliefs);
		}
		//appends worksheet to the page
		APP.setGoalWorksheet($view, $worksheet);	
	});
 
	//attach function to document on page showing, targeting #edit element to display the hash (contents)
	$(document).on('pageshowing', '#edit', function(e, ui) {
		if (APP.Debug) console.log("showing #edit!");
		//if there is no previous page send them back to list
        var $page = $(ui.prevPage);
		if ($page.length == 0) {
			$(":mobile-pagecontainer").pagecontainer("change", "#list", { transition: "none" });
			return;
		}
        
        // added check - if we are coming from the #save dialog, don't reload the contents of the page UNLESS the worksheet was just freshly saved
        if (APP.IsFreshlySaved || $page.attr('id') != 'save') {
            // reset IsFreshlySaved
            APP.IsFreshlySaved = false;
            if (APP.Debug) console.log(APP.CurrentGoalHash);
            var $edit = $(ui.toPage);
            var data = Storage.getWorksheetData(APP.CurrentGoalHash);
            var $worksheet = APP.newGoalWorksheet(data.title, data.outcome);
            // add the key to the save button, so we know which goal is being saved
            $edit.find('[data-goal-edit="true"]').attr('data-goal-old', APP.CurrentGoalHash);
            // create a new section
            for (var i = 0; i < data.sections.length; i++) {
                APP.addFullSection($worksheet, i+1, data.sections[i].priority, data.sections[i].actions, data.sections[i].beliefs);
            }
            //appends worksheet to the page
            APP.setGoalWorksheet($edit, $worksheet);
        }
		
	});
	
    $(document).on('pageshowing', '#edit-template', function(e, ui) {        
        if (APP.Debug) console.log("showing #edit-template!");
		//if there is no previous page send them back to list
        var $page = $(ui.prevPage);
		if ($page.length == 0) {
			$(":mobile-pagecontainer").pagecontainer("change", "#templates", { transition: "none" });
			return;
		}
        // added check - if we are coming from the #save dialog, don't reload the contents of the page
        if ($page.attr('id') != 'save') {
            if (APP.Debug) console.log(APP.CurrentTemplate);
            var $edit = $(ui.toPage);
            var data = Storage.getTemplateData(APP.CurrentTemplate);
            var $worksheet = APP.newGoalWorksheet(data.title, data.outcome);
            // add the key to the save button, so we know which goal is being saved
            //$edit.find('[data-goal-edit="true"]').attr('data-goal-old', APP.CurrentGoalHash);
            // create a new section
            for (var i = 0; i < data.sections.length; i++) {
                APP.addFullSection($worksheet, i+1, data.sections[i].priority, data.sections[i].actions, data.sections[i].beliefs);
            }
            //appends worksheet to the page
            APP.setGoalWorksheet($edit, $worksheet);
        }
    });
    
	// on pageshowing #list, rebuild the saved goal list
	$(document).on("pageshowing", "#list", function(e, ui) {
		if (APP.Debug) console.log("showing #list!");
		// grab the #list page
		var $list = $(ui.toPage);
		// get a list of the worksheet key
		var worksheets = APP.Storage.getWorksheetList();
		// start the list output
		var html = '<ul data-role="listview" data-goal="worksheetlist">';
		// if there are worksheets
		if (worksheets.length > 0) {
			// make a li for each worksheet
			for (var i = 0; i < worksheets.length; i++) {
				html += '<li><a href="#view" class="ui-btn ui-icon-right ui-btn-icon-right ui-nodisc-icon" data-goal="' + worksheets[i][0] + '">' + worksheets[i][1] + '</a></li>';
			}
		}
		// else put the sole no worksheets entry
		else {
			html += '<li>No saved goals.</li>';
		}
		// end the list output
		html += '</ul>';
		// replace the existing list with this new one
		$list
			.find("ul:jqmData(goal='worksheetlist')")
			.replaceWith($(html).listview().enhanceWithin());
	});
    
    // on pageshowing #list, rebuild the saved goal list
	$(document).on("pageshowing", "#remove", function(e, ui) {
		if (APP.Debug) console.log("showing #remove!");
		// grab the #list page
		var $remove = $(ui.toPage);
		// get a list of the worksheet key
		var worksheets = APP.Storage.getWorksheetList();
		// start the list output
		var html = '<ul data-role="listview" data-goal="worksheetlist">';
		// if there are worksheets
		if (worksheets.length > 0) {
			// make a li for each worksheet
			for (var i = 0; i < worksheets.length; i++) {
				html += '<li><a href="#" class="ui-btn ui-icon-remove ui-btn-icon-right ui-nodisc-icon" data-goal="' + worksheets[i][0] + '">' + worksheets[i][1] + '</a></li>';
			}
		}
		// else put the sole no worksheets entry
		else {
			html += '<li>No goals to remove.</li>';
		}
		// end the list output
		html += '</ul>';
		// replace the existing list with this new one
		$remove
			.find("ul:jqmData(goal='worksheetlist')")
			.replaceWith($(html).listview().enhanceWithin());
	});

})(jQuery);