// closure to prevent namespace polution
(function($) {
	// global application variable
	var APP = (window.APP = window.APP || {});
	// global storage functionality
	var Storage = APP.Storage = {};
	// worksheet items in storage
	Storage.getWorksheetKeys = function() {
	    // console.log("##########getWorksheetKeys");
	    if (window.external.ContainsKey(APP.KeysKey))
	        //if ( window.localStorage[APP.KeysKey])
	        //return JSON.parse(window.localStorage[APP.KeysKey]);
	        return JSON.parse(window.external.GetItem(APP.KeysKey));
		return [];
	};
	Storage.setWorksheetKeys = function(keys) {
        //console.log("##########setWorksheetKeys ", keys);
	    //window.localStorage[APP.KeysKey] = JSON.stringify(keys);
	    window.external.SetItem(APP.KeysKey, JSON.stringify(keys));
		//another way to write this:
        //window.localStorage.setItem(APP.KeysKey, JSON.stringify(keys));
	};
	Storage.saveWorksheetData = function(key, data) {
        //console.log("##########saveWorksheetData ", key, data);
	    //window.localStorage[key] = JSON.stringify(data);
	    window.external.SetItem(key, JSON.stringify(data));
		//another way to write this:
        //window.localStorage.setItem(key, JSON.stringify(data));
	};
	Storage.getWorksheetData = function(key) {
        //console.log("##########getWorksheetData", key);
	    //return JSON.parse(window.localStorage[key]);
	    return JSON.parse(window.external.GetItem(key));
	};
    Storage.getTemplateData = function(key) {
        return Storage.templates[key];
    };
    
    Storage.clearWorksheetData = function(key) {
        // attempt to remove given key from existing keys
		var existingKeys = Storage.getWorksheetKeys();
        for (var i = 0; i < existingKeys.length; i++) {
            // key found
            if (existingKeys[i] == key) {
                // remove key from existing keys
                existingKeys.splice(i, 1);
                // refresh the keys
                Storage.setWorksheetKeys(existingKeys);
                // remove the actual worksheet
                //window.localStorage.removeItem(key);
                window.external.RemoveItem(key);
                return;
            }
        }
	};
    
	Storage.clearAll = function() {
        //console.log("##########clearAll");
		var existingKeys = Storage.getWorksheetKeys();
		for (var i = 0; i < existingKeys.length; i++) {
		    //window.localStorage.removeItem(existingKeys[i]);
		    window.encodeURI.RemoveItem(existingKeys[i]);
		}
		//window.localStorage.removeItem(APP.KeysKey);
		window.external.RemoveItem(APP.KeysKey);
	};
	Storage.getWorksheetList = function() {
		var worksheets = [];
		// get existing keys
		var existingKeys = Storage.getWorksheetKeys();
		for (var i = 0; i < existingKeys.length; i++) {
			var currentWorksheet = Storage.getWorksheetData(existingKeys[i]);
			worksheets.push([existingKeys[i], currentWorksheet.title]);
		}
		return worksheets;
	};
	// validation functionality
	Storage.validateWorksheetData = function(data) {
		// check the worksheet data here for any validation needed
		if ((!("title" in data)) || (data.title === "")) return "You must enter a Goal Title.";
		if ((!("outcome" in data)) || (data.outcome === "")) return "You must enter a Goal Outcome.";
		// if everything checks out, return empty string (no error)
		return "";
	};
	
	// saving functionality
	Storage.trySaveWorksheetData = function(data) {
		// get existing worksheet keys
		var existingKeys = Storage.getWorksheetKeys(),
			newKey = "goals." + MD5(data.title);
		// if the current key already exists
		if ($.inArray(newKey, existingKeys) !== -1) {
			return "A goal worksheet with this title already exists. Select 'Continue Editing' and enter a different Goal Title.";
		}
		// save the worksheet data at the new key
		Storage.saveWorksheetData(newKey, data);
		// add the new key to the worksheet keys
		existingKeys.push(newKey);
        APP.CurrentGoalHash = newKey;
		Storage.setWorksheetKeys(existingKeys);
		// return empty string (no error)
		return "";
	};
    
    //editing functionality
    Storage.tryOverwriteWorksheetData = function(data, oldKey) {
        // get existing worksheet keys
		var existingKeys = Storage.getWorksheetKeys(),
			newKey = "goals." + MD5(data.title);
        
        if (oldKey == newKey) {
            Storage.saveWorksheetData(newKey, data);
        } else if ($.inArray(newKey, existingKeys) !== -1) {
			return "A goal worksheet with this title already exists. Select 'Continue Editing' and enter a different Goal Title.";
		} else {
            //TODO: could replace the item being removed with a call to clearWorksheetData
            //title changed. save new key/value, remove old key/value:
            // remove the old key
            for (var i = 0; i < existingKeys.length; i++) {
                if (existingKeys[i] == oldKey) {
                    existingKeys.splice(i, 1);
                    break;
                }
            }
            // add the new key
            existingKeys.push(newKey);
            // save the new keys
            Storage.setWorksheetKeys(existingKeys);
            // save the new worksheet
			Storage.saveWorksheetData(newKey, data);
            // remove old item
            //window.localStorage.removeItem(oldKey);
			window.external.RemoveItem(oldKey);
            // set the current worksheet
            APP.CurrentGoalHash = newKey;
        }
        return "";
    };
   
    
    
    
    
    
    
    //> templates
    
    Storage.templates = [
        {
            title: 'Earn College Degree',
            outcome: 'To earn my associate’s degree.',
            sections: [
                {
                    priority: 'I will be a more confident student',
                    actions: [
                    'I review my past successes before each class.',
                    'I would like to score high on every test.',
                    'I need to be successful from the beginning.'
                    ],
                    beliefs: [
                    'The best students are confident.',
                    'I am a top-notch scholar in every way.',
                    'I always exceed the teacher’s expectations.'
                    ],
                },
                {
                    priority: 'I gather support around me.',
                    actions: [
                    'I update family and friends once a week.',
                    'I reward my spouse for watching the kids.',
                    'I use the tutoring program 2 times a week. '
                    ],
                    beliefs: [
                    'When I win, my family wins',
                    'Don’t be a loser.',
                    'Fight for yourself, fight for your family.'
                    ],
                }
            ]
        },
        {
            title: 'Graduate Ranger School',
            outcome: 'Graduate from Ranger School this year.',
            sections: [
                {
                    priority: 'Steadily increase my physical fitness',
                    actions: [
                    'I lift shoulders and back twice a week for 40 minutes, varying the intensity of each work-out.',
                    'I run five days a week, averaging a 7:15 pace for at least 40 miles per week.',
                    'I swim at least once a week for 800 meters.',
                    'I march with a 35-pound pack six or more miles weekly.'
                    ],
                    beliefs: [
                    'I shoulder more than my share of the task, whatever it may be, more than 100 percent.',
                    '“If you want to shine, you got to burn.” '
                    ],
                },
                {
                    priority: 'I practice Infantry-based Leadership.',
                    actions: [
                    'I read a section from FM 3-21.8 at least three times weekly. ',
                    'I practice writing OPORDs.',
                    'I carry the Ranger Handbook with me at all times.'
                    ],
                    beliefs: [
                    'I am a technically and tactically competent team player making my battle buddies successful. ',
                    'I know the basics of patrolling. I know the battle drills. I know troop leading procedures. Gallantly, I show the world I am a specially selected and well-trained Soldier.'
                    ],
                }
            ]
        },
        {
            title: 'Earn Drill Sergeant of the Year',
            outcome: 'Best Drill Sergeant in my Brigade.',
            sections: [
                {
                    priority: 'I am an expert marksman and trainer',
                    actions: [
                    'I attend AWG CATC training.',
                    'I read all weapons FMs and TMs.',
                    'I become the SME on the Engagement Skills Trainer.',
                    'I continually research close quarters battle, refine shooting techniques, and develop new training techniques.'
                    ],
                    beliefs: [
                    'I am the go-to guy and a team player.',
                    'I shoot expert every time.',
                    'I am the bests shot in the battalion.'
                    ],
                },
                {
                    priority: 'I lead Soldiers from the front',
                    actions: [
                    'I conduct PT with Soldiers every day. ',
                    'I prepare fully for every class I teach.',
                    'I use combat experience to motivate and to make training realistic and relevant.'
                    ],
                    beliefs: [
                    'I help my Soldiers set goals, and I mentor them to achieve their goals.',
                    'I am a dynamic leader and trainer.',
                    'I am a compassionate leader with Soldiers who are struggling.'
                    ],
                }
            ]
        },
        {
            title: 'Earn Promotion',
            outcome: 'I am promoted to SSG by the end of the year.',
            sections: [
                {
                    priority: 'Increase my military and civilian education',
                    actions: [
                    'I attend WTC in March.',
                    'I graduate WLC NLT mid April.',
                    'I complete one correspondence course a week.',
                    'I speak to an education counselor this week. '
                    ],
                    beliefs: [
                    'I research different colleges and degree programs this weekend. ',
                    'I enroll in a criminal justice program ASAP.',
                    'I take at least two college courses each semester.',
                    '“Study hard today. Play hard tomorrow.”'
                    ],
                },
                {
                    priority: 'Increase physical fitness',
                    actions: [
                    'I run at least two miles on Tuesdays, Thursdays and Saturdays after work.',
                    'I do an upper body workout on Mondays. ',
                    'I do a lower body workout on Wednesdays.',
                    'I do an abdominal workout on Fridays.'
                    ],
                    beliefs: [
                    'I limit fast food to once a month.',
                    'I drink at least six bottles of water daily for hydration.'
                    ],
                }
            ]
        },
        {
            title: 'Development Plan',
            outcome: 'My development plan for success.',
            sections: [
                {
                    priority: 'My body is healthy',
                    actions: [
                        'I attend three exercise classes a week to help release stress.',
                        'I eat a well-balanced diet including whole grains and four servings of fruits and vegetables each day.',
                        'I follow a sleep routine consisting of eight hours each night.',
                        'I try a new type of fun exercise one day per week.'
                    ],
                    beliefs: [
                        'I become healthier, stronger, and empowered.',
                        'I am healthier than I was yesterday.',
                        'This is my time!'
                    ],
                },
                {
                    priority: 'My mind is resilient',
                    actions: [
                        'I set and evaluate goals as new challenges arise. ',
                        'I practice relaxation with complete focus and intention daily.',
                        'I use meaningful cues to help refocus my priorities and maintain a strong sense of self.',
                        'I reach out to my social support daily.'
                    ],
                    beliefs: [
                        'My goals and cues make me more resilient every day.',
                        'I am calm and able to tackle any challenge.',
                        'I am a strong support to my family and community.'
                    ],
                }
			]
		},
		{
            title: 'Score 300 APFT',
            outcome: 'Score  300 on my APFT',
            sections: [
                {
                    priority: 'I physically and mentally prepare for the APFT',
                    actions: [
                    'I eat a big meal two nights before the test and eat a more modest meal the night before, focusing on plain foods, complex carbs and hydration.',
                    'The night before, I carve out five to 10 minutes to visualize myself executing my APFT perfectly and achieving my goals.',
                    'I do a pre-combat check of all needed PT gear the night before so that I feel ready and relaxed in the morning.'
                    ],
                    beliefs: [
                    'I am fully equipped to crush my previous APFT score – I’ve done the hard work and now I go “all in” and reap the rewards.',
                    'Pain has become my new benchmark of success. The more it hurts the better I am doing.',
                    '“It’s not the will to win, but the will to prepare that makes the difference, and I win the battle before it happens.”'
                    ],
                },
                {
                    priority: 'I physically and mentally warm-up for the APFT',
                    actions: [
                    'I prepare my muscles by flexing and stretching my chest, arms and abs when standing in line.',
                    'I do an easy jog and some light stretching on the infield of the track before the run. I make sure I’m sweating before the run begins.',
                    'While warming up physically, I prepare my mind by creating positive images of success, accompanied by motivating and confident self-talk and a lighthearted intensity.'
                    ],
                    beliefs: [
                    'I would rather be confident than comfortable. I embrace my nerves and steer my butterflies to fly in formation.',
					'I pay my dues now to reap the rewards later. Short-term pain leads to long-term satisfaction.',
                    'I start strong and finish stronger. The longer the workout, the better I get.'
                    ],
                },
				{
                    priority: 'I  execute  my physical and mental  APFT plan.',
                    actions: [
                    'Push-ups and sit-ups. I relax and let gravity do the work on the way down.',
                    'I explode from the ground to halfway, and my momentum finishes each rep.',
                    'Run: I focus on running tall with my head still and level, eyes locked on my next target, arms loose and moving forward and backward.',
					'I use my cues. “Fire the pistons” for push-ups, “crack the whip” for sit-ups and “glide the stride” for my two mile run.'
                    ],
                    beliefs: [
                    'I work hard during events and recover using my time in between events to recharge and refocus.',
					'I choose to suffer for 12 minutes rather than regret giving into a comfortable 14- minute run.',
					'Short-term pain leads to long-term satisfaction.',
                    'My blistering pace and silky stride make a greyhound feel inadequate.'
                    ]
                }
            ]
        }
	];
})(jQuery);




















